// ************************************************************************** //
//
//  BornAgain: simulate and fit scattering at grazing incidence
//
//! @file      coregui/Models/ApplicationModels.h
//! @brief     Defines class holding all application models
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2016
//! @authors   Scientific Computing Group at MLZ Garching
//! @authors   Céline Durniak, Marina Ganeva, David Li, Gennady Pospelov
//! @authors   Walter Van Herck, Joachim Wuttke
//
// ************************************************************************** //
#include "ApplicationModels.h"
#include "GUIObjectBuilder.h"
#include "DocumentModel.h"
#include "MaterialModel.h"
#include "MaterialSvc.h"
#include "InstrumentModel.h"
#include "SampleModel.h"
#include "JobModel.h"
#include "IconProvider.h"
#include "SampleBuilderFactory.h"
#include "JobItem.h"
#include "SimulationOptionsItem.h"
#include "IntensityDataIOFactory.h"
#include "Histogram2D.h"
#include "IntensityDataItem.h"
#include <QMessageBox>

ApplicationModels::ApplicationModels(QObject *parent)
    : QObject(parent)
    , m_documentModel(0)
    , m_materialModel(0)
    , m_materialSvc(0)
    , m_instrumentModel(0)
    , m_sampleModel(0)
    , m_jobModel(0)
{
    createModels();
//    createTestSample();
//    createTestJob();

//    SessionItem *multilayer = m_sampleModel->insertNewItem(Constants::MultiLayerType);
//    SessionItem *layer = m_sampleModel->insertNewItem(Constants::LayerType, multilayer->index());

//    SampleModel * mmm = m_sampleModel->createCopy();

}

ApplicationModels::~ApplicationModels()
{
    delete m_materialSvc;
}

DocumentModel *ApplicationModels::documentModel()
{
    return m_documentModel;
}

MaterialModel *ApplicationModels::materialModel()
{
    return m_materialModel;
}

InstrumentModel *ApplicationModels::instrumentModel()
{
    return m_instrumentModel;
}

SampleModel *ApplicationModels::sampleModel()
{
    return m_sampleModel;
}

JobModel *ApplicationModels::jobModel()
{
    return m_jobModel;
}

ObsoleteFitModel *ApplicationModels::fitModel()
{
    return 0;
}

//! reset all models to initial state
void ApplicationModels::resetModels()
{
    m_documentModel->clear();
    m_documentModel->insertNewItem(Constants::SimulationOptionsType);

    m_materialModel->clear();
    m_materialModel->addMaterial("Default", 1e-3, 1e-5);
    m_materialModel->addMaterial("Air", 0.0, 0.0);
    m_materialModel->addMaterial("Particle", 6e-4, 2e-8);
    m_materialModel->addMaterial("Substrate", 6e-6, 2e-8);

    m_sampleModel->clear();

    m_jobModel->clear();

    m_instrumentModel->clear();
    SessionItem *instrument = m_instrumentModel->insertNewItem(Constants::InstrumentType);
    instrument->setItemName("Default GISAS");
    m_instrumentModel->insertNewItem(Constants::DetectorType, m_instrumentModel->indexOfItem(instrument));
    m_instrumentModel->insertNewItem(Constants::BeamType, m_instrumentModel->indexOfItem(instrument));
}

//! creates and initializes models, order is important
void ApplicationModels::createModels()
{
    createDocumentModel();

    createMaterialModel();

    createSampleModel();

    createInstrumentModel();

    createJobModel();

    resetModels();
}

void ApplicationModels::createDocumentModel()
{
    delete m_documentModel;
    m_documentModel = new DocumentModel(this);
    connectModel(m_documentModel);
}

void ApplicationModels::createMaterialModel()
{
    delete m_materialModel;
    m_materialModel = new MaterialModel(this);
    connectModel(m_materialModel);
    m_materialSvc = new MaterialSvc(m_materialModel);
}

void ApplicationModels::createSampleModel()
{
    Q_ASSERT(m_materialModel);
    delete m_sampleModel;
    m_sampleModel = new SampleModel(this);
    connectModel(m_sampleModel);
    connect(m_materialModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
            m_sampleModel, SLOT(onMaterialModelChanged(QModelIndex,QModelIndex)));
}

void ApplicationModels::createJobModel()
{
    delete m_jobModel;
    m_jobModel = new JobModel(this);
    connectModel(m_jobModel);
}

void ApplicationModels::createInstrumentModel()
{
    delete m_instrumentModel;
    m_instrumentModel = new InstrumentModel(this);
    connectModel(m_instrumentModel);
    m_instrumentModel->setIconProvider(new IconProvider());
}

void ApplicationModels::createTestSample()
{
    SampleBuilderFactory factory;
    const std::unique_ptr<ISample> P_sample(factory.createSample("CylindersAndPrismsBuilder"));

    GUIObjectBuilder guiBuilder;
    guiBuilder.populateSampleModel(m_sampleModel, *P_sample);

//    SimulationRegistry simRegistry;
//    const std::unique_ptr<GISASSimulation> simulation(simRegistry.createSimulation("RectDetectorPerpToReflectedBeamDpos"));
    //    guiBuilder.populateInstrumentModel(m_instrumentModel, *simulation);
}

void ApplicationModels::createTestJob()
{
    SimulationOptionsItem *optionsItem = m_documentModel->getSimulationOptionsItem();
    optionsItem->setRunPolicy(Constants::JOB_RUN_IN_BACKGROUND);

    JobItem *jobItem = m_jobModel->addJob(
                m_sampleModel->getMultiLayerItem(),
                m_instrumentModel->getInstrumentItem(),
                optionsItem);

    IHistogram *data = IntensityDataIOFactory::readIntensityData("/home/pospelov/development/BornAgain/temp/Untitled12/data_job1_0.int");
    dynamic_cast<IntensityDataItem*>(jobItem->getItem(JobItem::T_REALDATA))
            ->setOutputData(data->createOutputData());
    jobItem->setItemValue(JobItem::P_WITH_FITTING, true);


    m_jobModel->runJob(jobItem->index());

}

void ApplicationModels::disconnectModel(SessionModel *model)
{
    if(model) {
        disconnect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this,
                   SIGNAL(modelChanged()));
        disconnect(model, SIGNAL(rowsRemoved(QModelIndex, int, int)), this,
                   SIGNAL(modelChanged()));
        disconnect(model, SIGNAL(rowsInserted(QModelIndex, int,int)), this,
                   SIGNAL(modelChanged()));
    }
}

void ApplicationModels::connectModel(SessionModel *model)
{
    if(model) {
        connect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this,
                   SIGNAL(modelChanged()), Qt::UniqueConnection);
        connect(model, SIGNAL(rowsRemoved(QModelIndex, int, int)), this,
                   SIGNAL(modelChanged()), Qt::UniqueConnection);
        connect(model, SIGNAL(rowsInserted(QModelIndex, int,int)), this,
                   SIGNAL(modelChanged()), Qt::UniqueConnection);
    }
}

